@@ -464,7 +464,7 @@ extern void afs_osi_Invisible(void);
 extern void afs_osi_RxkRegister(void);
 extern void afs_osi_MaskSignals(void);
 extern void afs_osi_UnmaskRxkSignals(void);
-extern void *afs_osi_Alloc(size_t x);
+extern void *afs_osi_Alloc_debug(size_t x, char *func, int line);
 #ifndef afs_osi_Alloc_NoSleep
 extern void *afs_osi_Alloc_NoSleep(size_t x);
 #endif
@@ -685,7 +685,7 @@ extern struct srvAddr *afs_srvAddrs[NSERVERS];
 extern struct server *afs_servers[NSERVERS];
 extern int afs_totalServers;
 extern struct server *afs_setTimeHost;
-extern struct server *afs_FindServer(afs_int32 aserver, ushort aport,
+extern struct server *afs_FindServer(afs_int32 aserver, afs_uint16 aport,
 				     afsUUID * uuidp, afs_int32 locktype);
 extern struct server *afs_GetServer(afs_uint32 * aserver, afs_int32 nservers,
 				    afs_int32 acell, u_short aport,
@@ -711,6 +711,7 @@ extern int afsi_SetServerIPRank(struct srvAddr *sa, afs_int32 addr,
 void afsi_SetServerIPRank(struct srvAddr *sa, struct in_ifaddr *ifa);
 #endif
 #endif
+extern int afs_HaveCallBacksFrom(struct server *aserver);
 
 
 
@@ -855,7 +856,7 @@ extern void afs_ProcessFS(register struct vcache *avc,
 			  struct vrequest *areq);
 extern struct afs_cbr *afs_AllocCBR(void);
 extern int afs_FreeCBR(register struct afs_cbr *asp);
-extern int afs_RemoveVCB(register struct VenusFid *afid);
+extern void afs_RemoveVCB(register struct VenusFid *afid);
 extern void afs_FlushActiveVcaches(register afs_int32 doflocks);
 extern int afs_WriteVCache(register struct vcache *avc,
 			   register struct AFSStoreStatus *astatus,
